/*========================================================================
  Function Viewer: main module
  ---------------
  This program is distributed under the GPL license.
==========================================================================*/

#define DEFAULT_WINDOW_WIDTH 800
#define DEFAULT_WINDOW_HEIGHT 600
#define PROGRAM_NAME "Function Viewer 1.0"

#include "Function.hh"
#include "Visualizer.hh"

#include <vector>
#include <cstdio>
#include <gtk/gtk.h>

class TopLevelUI
{
    GtkWidget* mainWindow;
    GtkWidget* vContainer;
    GtkWidget* buttonBar;
    Visualizer* visualizer;
    std::vector<Function*> functions;
    std::vector<GtkWidget*> buttons;

    template<typename CBFunc>
    void createButton(const char* name, CBFunc cb)
    {
        GtkWidget* button = gtk_button_new_with_mnemonic(name);
        g_signal_connect(GTK_OBJECT(button), "clicked",
                         G_CALLBACK(cb), this);
        gtk_box_pack_start(GTK_BOX(buttonBar), button, FALSE, FALSE, 0);
        buttons.push_back(button);
    }

    void setSensitive(int buttonNumber, gboolean sensitive)
    {
        gtk_widget_set_sensitive(buttons[buttonNumber], sensitive);
    }

 public:
    TopLevelUI()
    {
        // Main top level window:
        mainWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
        gtk_window_set_default_size(GTK_WINDOW(mainWindow),
                                    DEFAULT_WINDOW_WIDTH,
                                    DEFAULT_WINDOW_HEIGHT);
        gtk_window_set_title(GTK_WINDOW(mainWindow), PROGRAM_NAME);
        g_signal_connect(GTK_OBJECT(mainWindow), "destroy",
                         G_CALLBACK(gtk_main_quit), NULL);

        // Vertical widget container:
        vContainer = gtk_vbox_new(FALSE, 0);
        gtk_container_add(GTK_CONTAINER(mainWindow), vContainer);

        // Toolbar at the top of the window:
        buttonBar = gtk_hbox_new(FALSE, 0);
        gtk_box_pack_start(GTK_BOX(vContainer), buttonBar, FALSE, FALSE, 0);

        // Create buttons for the toolbar:
        createButton("_Add function", addFunctionCallback);
        createButton("_Remove function", removeFunctionCallback);
        setSensitive(1, FALSE);

        // The visualizer area:
        visualizer = new Visualizer(mainWindow, buttonBar);
        gtk_box_pack_start(GTK_BOX(vContainer), visualizer->getWidget(),
                           TRUE, TRUE, 0);

        // A function:
        functions.push_back(new Function(1));
        gtk_box_pack_start(GTK_BOX(vContainer), functions.back()->getWidget(),
                           FALSE, FALSE, 0);

        visualizer->addFunction(functions.back());

        gtk_widget_show_all(mainWindow);
    }

    ~TopLevelUI()
    {
        for(unsigned i = 0; i < functions.size(); ++i)
            delete functions[i];
        delete visualizer;
    }


 private:
    static void addFunctionCallback(GtkToolButton*, gpointer data)
    {
        TopLevelUI* instance = reinterpret_cast<TopLevelUI*>(data);

        if(instance->functions.size() == 99) return;

        Function* newFunc = new Function(instance->functions.size()+1);
        instance->functions.push_back(newFunc);

        // Connect the earlier functions to this one:
        char fName[4];
        for(unsigned i = 0; i < instance->functions.size()-1; ++i)
        {
            std::sprintf(fName, "f%u", i+1);
            newFunc->getParser().AddFunction
                (fName, instance->functions[i]->getParser());
        }

        // Add the function to the visualizer and the main window:
        instance->visualizer->addFunction(instance->functions.back());
        gtk_box_pack_start(GTK_BOX(instance->vContainer),
                           instance->functions.back()->getWidget(),
                           FALSE, FALSE, 0);

        instance->setSensitive(1, TRUE);
        gtk_widget_show_all(instance->mainWindow);
    }

    static void removeFunctionCallback(GtkToolButton*, gpointer data)
    {
        TopLevelUI* instance = reinterpret_cast<TopLevelUI*>(data);

        if(instance->functions.size() > 1)
        {
            delete instance->functions.back();
            instance->functions.pop_back();

            if(instance->functions.size() == 1)
                instance->setSensitive(1, FALSE);
        }
    }

    TopLevelUI(const TopLevelUI&);
    TopLevelUI& operator=(const TopLevelUI&);
};

int main(int argc, char* argv[])
{
    gtk_init(&argc, &argv);
    TopLevelUI ui;
    gtk_main();
}
