/*========================================================================
  Function Viewer: function module
  ---------------
  This program is distributed under the GPL license.
==========================================================================*/

#include "Function.hh"

#include <cstdio>
#include <cstdlib>

namespace
{
    const int FC = 0xFFFF, HC = 0x8000;

    const GdkColor DefaultColors[8] =
    {
        { 0, 0, 0, 0 }, { 0, FC, 0, 0 }, { 0, 0, FC, 0 }, { 0, 0, 0, FC },
        { 0, HC, HC, 0 }, { 0, 0, HC, HC }, { 0, HC, 0, HC }, { 0, HC, HC ,HC }
    };

    GdkColor getFunctionDefaultColor(unsigned functionNumber)
    {
        if(functionNumber < 8) return DefaultColors[functionNumber];
        GdkColor c;
        c.red = (std::rand()%255)*256;
        c.green = (std::rand()%255)*256;
        c.blue = (std::rand()%255)*256;
        return c;
    };
};

// Contructor and destructor
//-----------------------------------------------------------------------
Function::Function(unsigned functionNumber):
    container(gtk_hbox_new(FALSE, 0)),
    textField(gtk_entry_new()),
    callbackObject(0),
    validFunction(false)
{
    // Create check button labeled with the function name:
    char fName[8];
    std::sprintf(fName, "f%u(x)=", functionNumber%100);
    checkButton = gtk_check_button_new_with_label(fName);

    // Set a default color for the function:
    functionColor = getFunctionDefaultColor(functionNumber-1);
    colorSelectButton = gtk_color_button_new_with_color(&functionColor);

    // Populate the hbox:
    gtk_box_pack_start(GTK_BOX(container), checkButton, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(container), textField, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(container), colorSelectButton, FALSE, FALSE, 0);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkButton), TRUE);
    gtk_widget_grab_focus(textField);

    // Connect the necessary signals:
    g_signal_connect(GTK_OBJECT(container), "unrealize",
                     G_CALLBACK(destroyCallback), this);
    g_signal_connect(GTK_OBJECT(checkButton), "toggled",
                     G_CALLBACK(buttonToggleCallback), this);
    g_signal_connect(GTK_OBJECT(textField), "changed",
                     G_CALLBACK(textModifyCallback), this);
    g_signal_connect(GTK_OBJECT(colorSelectButton), "color-set",
                     G_CALLBACK(colorChangeCallback), this);

    // Initialize the parser:
    fParser.AddConstant("pi", 3.1415926535897932);
    // Set the parser internally to a valid state (ie. it can be evaluated)
    // so that if another parser calls this one, it won't break:
    fParser.Parse("0", "x");
}

Function::~Function()
{
    if(callbackObject) callbackObject->functionIsBeingDestroyed(this);
    if(container) gtk_widget_destroy(container);
}

void Function::registerCallback(Callback* cb)
{
    callbackObject = cb;
}


// Function parsing methods
//-----------------------------------------------------------------------
namespace
{
    const GdkColor RED_COLOR = { 0, 65535, 0, 0 };
};

void Function::textChanged()
{
    int res = fParser.Parse(gtk_entry_get_text(GTK_ENTRY(textField)), "x");

    if(res > -1)
    {
        gtk_widget_modify_text(textField, GTK_STATE_NORMAL, &RED_COLOR);
        validFunction = false;
    }
    else
    {
        validFunction = true;
        gtk_widget_modify_text(textField, GTK_STATE_NORMAL, NULL);
        if(callbackObject) callbackObject->functionHasChanged();
    }
}

void Function::checkButtonToggled()
{
    if(callbackObject) callbackObject->functionHasChanged();
}

void Function::colorChanged()
{
    gtk_color_button_get_color(GTK_COLOR_BUTTON(colorSelectButton),
                               &functionColor);
    if(callbackObject) callbackObject->functionHasChanged();
}


// GTK callback functions
//-----------------------------------------------------------------------
gboolean Function::destroyCallback(GtkWidget*, gpointer data)
{
    Function* instance = reinterpret_cast<Function*>(data);
    instance->container = 0;
    return FALSE;
}

void Function::textModifyCallback(GtkEditable*, gpointer data)
{
    Function* instance = reinterpret_cast<Function*>(data);
    instance->textChanged();
}

void Function::buttonToggleCallback(GtkToggleButton*, gpointer data)
{
    Function* instance = reinterpret_cast<Function*>(data);
    instance->checkButtonToggled();
}

void Function::colorChangeCallback(GtkColorButton*, gpointer data)
{
    Function* instance = reinterpret_cast<Function*>(data);
    instance->colorChanged();
}
